<?php

require_once(dirname(__FILE__) . '/../class/usermailboxconfig.class.php');
require_once(DOL_DOCUMENT_ROOT . "/comm/action/class/cactioncomm.class.php");
require_once(DOL_DOCUMENT_ROOT . "/comm/action/class/actioncomm.class.php");
require_once(DOL_DOCUMENT_ROOT . "/core/lib/images.lib.php");

/**
 * Wrap text to a given number of characters per line
 * but respect the mail quotation of replies messages (>).
 * Finally add another quotation level by prepending the lines
 * with >
 * Origine : roundcube/program/steps/mail/func.inc
 *
 * @param string Text to wrap
 * @param int    The line width
 * @return string The wrapped text
 */
function wrap_and_quote($text, $length = 72) {
    // Rebuild the message body with a maximum of $max chars, while keeping quoted message.
    $max = max(75, $length + 8);
    $lines = preg_split('/\r?\n/', trim($text));
    $out = '';
    $nb_lines = sizeof($lines);
    for ($i_line = 0; $i_line < $nb_lines; $i_line++) {
        $line = $lines[$i_line];
        // don't wrap already quoted lines
        if ($line[0] === '>')
            $line = '>' . rtrim($line);
        else if (mb_strlen($line) > $max) {
            $newline = '';
            $tablines = explode("\n", wordwrap($line, $length - 2));
            $nb_tablines = sizeof($tablines);
            for ($i_tabline = 0; $i_tabline < $nb_tablines; $i_tabline++) {
                $l = $tablines[$i_tabline];
                if (strlen($l))
                    $newline .= '> ' . $l . "\n";
                else
                    $newline .= ">\n";
            }
            $line = rtrim($newline);
        }
        else
            $line = '> ' . $line;

        // Append the line
        $out .= $line . "\n";
    }

    return rtrim($out, "\n");
}

/**
 * Function to create base body to reply mail
 * 
 * Origine : roundcube/program/steps/mail/compose.inc
 * 
 * @param type $header
 * @param type $body
 * @param type $bodyIsHtml
 * @return type
 */
function create_reply_body($header, $body, $bodyIsHtml) {
    global $langs;

    // build reply prefix
    $from = $header->from;
    $prefix = '<br />' . $langs->trans('mailreplyinto') . "<br />";
    $prefix .= $langs->trans('sender') . ' : ' . $from[0]->mailbox . '@' . $from[0]->host . "<br />";
    $prefix .= $langs->trans('datesend') . ' : ' . date('d/m/Y H:i',
                    strtotime($header->date)) . ' (' . $header->date . ') ' . "<br />";
    $prefix .= $langs->trans('toadress') . ' : ' . $header->toadress . "<br /><br />";

    if (!$bodyIsHtml) {
        $body = preg_replace('/\r?\n/', "\n", $body);
        $body = trim($body, "\n");

        // soft-wrap and quote message text
        $body = wrap_and_quote($body);

        $prefix .= "\n";
        $suffix = '';

        $prefix = '__SIGNATURE__' . $prefix;
        $prefix = "\n\n\n" . $prefix;
    }
    else {
        // build reply (quote content)
        $prefix = '<p>' . $prefix . "</p>\n";
        $prefix .= '<blockquote>';
        $prefix = '<br>__SIGNATURE__' . $prefix;
        $suffix = '</blockquote>';
    }
    if (empty($conf->global->FCKEDITOR_ENABLE_MAIL))
        return dol_string_nohtmltag(str_replace("<br />", '\n',
                        str_replace("<br>", '\n', $prefix . $body . $suffix)));

    return $prefix . $body . $suffix;
}

function store_email_into_folder($msg, $folder = 'SentFromDolibarr') {
    global $user, $db;
    $mailboxconfig = new Usermailboxconfig($db);
    $mailboxconfig->fetch_from_user($user->id);

    $user->mailbox_imap_login = $mailboxconfig->mailbox_imap_login;
    $user->mailbox_imap_password = $mailboxconfig->mailbox_imap_password;
    $user->mailbox_imap_host = $mailboxconfig->mailbox_imap_host;
    $user->mailbox_imap_port = $mailboxconfig->mailbox_imap_port;
    $user->mailbox_imap_ssl = $mailboxconfig->mailbox_imap_ssl;
    $user->mailbox_imap_ssl_novalidate_cert = $mailboxconfig->mailbox_imap_ssl_novalidate_cert;
    $user->mailbox_imap_ref = $mailboxconfig->get_ref();
    $user->mailbox_imap_connector_url = $mailboxconfig->get_connector_url();

    $mbox = imap_open($user->mailbox_imap_connector_url . $folder,
            $user->mailbox_imap_login, $user->mailbox_imap_password);

    $check = imap_check($mbox);
    $before = $check->Nmsgs;

    $result = imap_append($mbox,
            $user->mailbox_imap_connector_url . $folder
            , $msg
    );

    $check = imap_check($mbox);
    $after = $check->Nmsgs;

    if ($result == FALSE) {
        if (imap_createmailbox($mbox,
                        imap_utf7_encode($user->mailbox_imap_ref . $folder))) {
            $mbox = imap_open($user->mailbox_imap_connector_url . $folder,
                    $user->mailbox_imap_login, $user->mailbox_imap_password);

            $check = imap_check($mbox);
            $before = $check->Nmsgs;

            $result = imap_append($mbox,
                    $user->mailbox_imap_connector_url . $folder
                    , $msg
            );

            $check = imap_check($mbox);
            $after = $check->Nmsgs;
        }
    }
    imap_close($mbox);
}

function serialize_mail($mail) {
    $str = date("Ymdhis", strtotime($mail->date)) . '_';
    $str .= trim(preg_replace('/<.*>|"/', '',
                            @iconv_mime_decode(imap_utf8($mail->from)))) . '_';
    $str .= trim(utf8_encode(@iconv_mime_decode(imap_utf8($mail->subject)))) . '_';
    $str .= $mail->size;

    return $str;
}

function sanitize_imap_folder($folder) {
    return str_replace("?;", "&nbsp;",
            utf8_encode(mb_convert_encoding($folder, "ISO_8859-1", "UTF7-IMAP")));
}

function updatekeyactioncomm($idaction, $key) {
    global $db;
    $sql = "UPDATE " . MAIN_DB_PREFIX . "actioncomm ";
    $sql.= " SET ref_ext = '" . $key . "'";
    $sql.= " WHERE id=" . $idaction;
    if ($db->query($sql)) {
        return true;
    }
    return false;
}

function getlisteliens($key) {
    global $conf, $user, $db, $langs;

    $sql = 'SELECT id, fk_element, elementtype FROM ' . MAIN_DB_PREFIX . 'actioncomm ';
    $sql .= 'WHERE ref_ext = "' . $key . '" ';

    $resql = $db->query($sql);
    if ($resql) {
        $num = $db->num_rows($resql);
        $i = 0;
        while ($i < $num) {
            $obj = $db->fetch_object($resql);
            $result[] = $obj;
            $i++;
        }
    }
    return $result;
}

function getnb_linked($mail) {
    global $conf, $user, $db, $langs;

    $serialized = sha1(serialize_mail($mail));

    $sql = 'SELECT COUNT(id) as linked FROM ' . MAIN_DB_PREFIX . 'actioncomm ';
    $sql .= 'WHERE ref_ext = "' . $serialized . '" ';

    $resql = $db->query($sql);
    if ($resql) {
        if ($db->num_rows($resql)) {
            $obj = $db->fetch_object($resql);
            return $obj->linked;
        }
    }
    return 0;
}

/**
 * Retourne le nombre de pièce(s) jointe(s) d'un mail
 * 
 * @param int $mail IUD du mail
 * @param string $mbox ImapStream retourné par la fonction imap_open
 * @return int Le nombre de pièces jointes
 */
function getnb_attachement($mail_uid, $mbox) {
    $detail_mail = imap_fetchstructure($mbox, $mail_uid, FT_UID);

    if ($detail_mail->ifdparameters)
        return 1;
    $nb_parts = count($detail_mail->parts);
    for ($i = 1; $i < $nb_parts; $i++)
        if ($detail_mail->parts[$i]->ifdparameters)
            return 1;
    return 0;
}

function getmsg($mbox, $mid) {
    $key = GETPOST('key');

    if (!is_dir(DOL_DATA_ROOT . '/dolimapmail'))
        dol_mkdir(DOL_DATA_ROOT . '/dolimapmail');

    if (!is_dir(DOL_DATA_ROOT . '/dolimapmail/tmp'))
        dol_mkdir(DOL_DATA_ROOT . '/dolimapmail/tmp');

    if (!is_dir(DOL_DATA_ROOT . '/dolimapmail/tmp/' . $key))
        dol_mkdir(DOL_DATA_ROOT . '/dolimapmail/tmp/' . $key);


    // input $mbox = IMAP stream, $mid = message id
    // output all the following:
    $htmlmsg = $plainmsg = $charset = '';
    $attachments = array();

    // add code here to get date, from, to, cc, subject...
    // BODY
    $s = imap_fetchstructure($mbox, $mid, FT_UID);
    if (!$s->parts)  // simple
        list($charset, $htmlmsg, $plainmsg, $attachments) = getpart($mbox, $mid,
                $s, 0, $charset, $htmlmsg, $plainmsg, $attachments);  // pass 0 as part-number
    else {  // multipart: cycle through each part
        foreach ($s->parts as $partno0 => $p)
            list($charset, $htmlmsg, $plainmsg, $attachments) = getpart($mbox,
                    $mid, $p, $partno0 + 1, $charset, $htmlmsg, $plainmsg,
                    $attachments);
    }
    return array($charset, $htmlmsg, $plainmsg, $attachments);
}

function getpart($mbox, $mid, $p, $partno, $charset, $htmlmsg, $plainmsg,
        $attachments) {
    $key = GETPOST('key');
    // $partno = '1', '2', '2.1', '2.1.3', etc for multipart, 0 if simple
    // DECODE DATA
    if ($p->encoding != 3 || $partno < 2)
        $data = ($partno) ?
                imap_fetchbody($mbox, $mid, $partno, FT_UID) : // multipart
                imap_body($mbox, $mid, FT_UID);  // simple
// Any part may be encoded, even plain text messages, so check everything.
    if ($p->encoding == 4)
        $data = quoted_printable_decode($data);
    elseif ($p->encoding == 3)
    //$data = base64_decode($data);
    // PARAMETERS
    // get all parameters, like charset, filenames of attachments, etc.
        $params = array();
    if ($p->parameters)
        foreach ($p->parameters as $x)
            $params[strtolower($x->attribute)] = $x->value;
    if ($p->dparameters)
        foreach ($p->dparameters as $x)
            $params[strtolower($x->attribute)] = $x->value;

    // ATTACHMENT
    // Any part with a filename is an attachment,
    // so an attached text file (type 0) is not mistaken as the message.
    if ($params['filename'] || $params['name']) {
        // filename may be given as 'Filename' or 'Name' or both
        $filename = trim(preg_replace('/<.*>|"\//', '',
                        @iconv_mime_decode(imap_utf8(($params['filename']) ? $params['filename'] : $params['name']))));
        // filename may be encoded, so see imap_mime_header_decode()
        $attachments[$filename] = $data;  // this is a problem if two files have same name

        $upload_dir = DOL_DATA_ROOT . '/dolimapmail/tmp/' . $key;

        $tFile = $upload_dir . "/" . $filename;
        if (!dol_is_file($tFile)) {
            $whandle = fopen($tFile, 'w');

            stream_filter_append($whandle, 'convert.base64-decode',
                    STREAM_FILTER_WRITE);

            imap_savebody($mbox, $whandle, $mid, $partno, FT_UID);

            fclose($whandle);
        }
    }

    // TEXT
    if ($p->type == 0 && $data) {
        // Messages may be split in different parts because of inline attachments,
        // so append parts together with blank row.
        if (strtolower($p->subtype) === 'plain')
            $plainmsg .= trim($data) . "\n\n";
        else
            $htmlmsg .= $data . "<br /><br />";
        $charset = $params['charset'];  // assume all parts are same charset
    }

    // EMBEDDED MESSAGE
    // Many bounce notifications embed the original message as type 2,
    // but AOL uses type 1 (multipart), which is not handled here.
    // There are no PHP functions to parse embedded messages,
    // so this just appends the raw source to the main message.
    elseif ($p->type == 2 && $data) {
        $plainmsg .= $data . "\n\n";
    }

    // SUBPART RECURSION
    if ($p->parts) {
        foreach ($p->parts as $partno0 => $p2)
            list($charset, $htmlmsg, $plainmsg, $attachments) = getpart($mbox,
                    $mid, $p2, $partno . '.' . ($partno0 + 1), $charset,
                    $htmlmsg, $plainmsg, $attachments);  // 1.2, 1.2.1, etc.
    }
    return array($charset, $htmlmsg, $plainmsg, $attachments);
}

/**
 * List urls of element -- COPY OF getElementUrl Dolibarr < 3.5
 *
 * @param 	int		$objectid		Id of record
 * @param 	string	$objecttype		Type of object
 * @param 	int		$withpicto		Picto to show
 * @param 	string	$option			More options
 * @return	void
 */
function dolimapmail_getElementUrl($objectid, $objecttype, $withpicto = 0,
        $option = '') {
    global $conf, $db;

    // Parse element/subelement (ex: project_task)
    $module = $element = $subelement = $objecttype;
    if (preg_match('/^([^_]+)_([^_]+)/i', $objecttype, $regs)) {
        $module = $element = $regs[1];
        $subelement = $regs[2];
    }

    $classpath = $element . '/class';

    // To work with non standard path
    if ($objecttype === 'facture' || $objecttype === 'invoice') {
        $classpath = 'compta/facture/class';
        $module = 'facture';
        $subelement = 'facture';
    }
    if ($objecttype === 'commande' || $objecttype === 'order') {
        $classpath = 'commande/class';
        $module = 'commande';
        $subelement = 'commande';
    }
    if ($objecttype === 'propal') {
        $classpath = 'comm/propal/class';
    }
    if ($objecttype === 'shipping') {
        $classpath = 'expedition/class';
        $subelement = 'expedition';
        $module = 'expedition_bon';
    }
    if ($objecttype === 'delivery') {
        $classpath = 'livraison/class';
        $subelement = 'livraison';
        $module = 'livraison_bon';
    }
    if ($objecttype === 'invoice_supplier') {
        $classpath = 'fourn/class';
    }
    if ($objecttype === 'order_supplier') {
        $classpath = 'fourn/class';
    }
    if ($objecttype === 'contract') {
        $classpath = 'contrat/class';
        $module = 'contrat';
        $subelement = 'contrat';
    }
    if ($objecttype === 'member') {
        $classpath = 'adherents/class';
        $module = 'adherent';
        $subelement = 'adherent';
    }
    if ($objecttype === 'cabinetmed_cons') {
        $classpath = 'cabinetmed/class';
        $module = 'cabinetmed';
        $subelement = 'cabinetmedcons';
    }
    if ($objecttype === 'supplier_proposal') {
        $classpath = 'supplier_proposal/class';
        $module = 'supplier_proposal';
        $subelement = 'supplier_proposal';
    }

    //print "objecttype=".$objecttype." module=".$module." subelement=".$subelement;

    $classfile = strtolower($subelement);
    $classname = ucfirst($subelement);
    if ($objecttype === 'invoice_supplier') {
        $classfile = 'fournisseur.facture';
        $classname = 'FactureFournisseur';
    }
    if ($objecttype === 'order_supplier') {
        $classfile = 'fournisseur.commande';
        $classname = 'CommandeFournisseur';
    }
    if ($objecttype === 'supplier_proposal') {
        $classname = 'SupplierProposal';
    }


    if (!empty($conf->$module->enabled)) {
        $res = dol_include_once('/' . $classpath . '/' . $classfile . '.class.php');
        if ($res) {
            $object = new $classname($db);
            $ret = $object->fetch($objectid);
            if ($ret > 0)
                return $object->getNomUrl($withpicto, $option);
        }
    }
}

function getMenuLeftId() {
    global $db;

    $sql = 'SELECT MIN(rowid) as id_menu FROM ' . MAIN_DB_PREFIX . 'menu WHERE type="left" AND module="dolimapmail" ';
    $resql = $db->query($sql);

    if ($resql) {
        $obj = $db->fetch_object($resql);
        if ($obj->id_menu > 0) {
            return $obj->id_menu;
        }
    }
    return -1;
}

/**
 *   Load all objects with filters.
 *   WARNING: This make a fetch on all records instead of making one request with a join.
 *
 *   @param		DoliDb	$db				Database handler
 *   @param		int		$socid			Filter by thirdparty
 * 	 @param		int		$fk_element		Id of element action is linked to
 *   @param		string	$elementtype	Type of element action is linked to
 *   @param		string	$filter			Other filter
 *   @param		string	$sortfield		Sort on this field
 *   @param		string	$sortorder		ASC or DESC
 *   @param		string	$limit			Limit number of answers
 *   @return	array or string			Error string if KO, array with actions if OK
 */
function getLinkedMails($db, $socid = 0, $fk_element = 0, $elementtype = '',
        $filter = '', $sortfield = 'a.datep', $sortorder = 'DESC', $limit = 0) {
    global $conf, $langs;

    $resarray = array();
    if ($conf->global->DOLIMAPMAIL_CODE_ACTIONCOMM)
        $cactioncomm_code = $conf->global->DOLIMAPMAIL_CODE_ACTIONCOMM;
    else
        $cactioncomm_code = "AC_OTH";

    $sql = "SELECT a.id";
    $sql.= " FROM " . MAIN_DB_PREFIX . "actioncomm as a";
    $sql.= " WHERE a.entity IN (" . getEntity('agenda') . ")";
    if (!empty($socid))
        $sql.= " AND a.fk_soc = " . $socid;
        $sql.= " AND a.code = '" . $cactioncomm_code."' ";
//        $sql.= " AND NOT ( a.ref_ext IS NULL ) ";
    if (!empty($elementtype)) {
        if ($elementtype == 'project')
            $sql.= ' AND a.fk_project = ' . $fk_element;
        else
            $sql.= " AND a.fk_element = " . (int) $fk_element . " AND a.elementtype = '" . $elementtype . "'";
    }
    if (!empty($filter))
        $sql.= $filter;
    if ($sortorder && $sortfield)
        $sql.=$db->order($sortfield, $sortorder);
    $sql.=$db->plimit($limit, 0);
    dol_syslog(get_class() . "::getLinkedMails", LOG_DEBUG);
    $resql = $db->query($sql);
    if ($resql) {
        $num = $db->num_rows($resql);

        if ($num) {
            for ($i = 0; $i < $num; $i++) {
                $obj = $db->fetch_object($resql);
                $actioncommstatic = new ActionComm($db);
                $actioncommstatic->fetch($obj->id);
                $resarray[$i] = $actioncommstatic;
            }
        }
        $db->free($resql);
        
        return $resarray;
    }
    else {
        return $db->lasterror();
    }
}
